@@ -1,6 +1,7 @@
 """
 ast_utils, bunch of helpers for converting input into ast.* input_str
 """
+
 import ast
 import pickle
 from ast import (
@@ -15,7 +16,10 @@ from ast import (
     Dict,
     Expr,
     FunctionDef,
+    Import,
+    ImportFrom,
     Index,
+    List,
     Load,
     Module,
     Name,
@@ -36,8 +40,9 @@ from contextlib import suppress
 from copy import deepcopy
 from importlib import import_module
 from inspect import isclass, isfunction
+from itertools import chain, filterfalse
 from json import dumps
-from operator import inv, neg, not_, pos
+from operator import attrgetter, contains, inv, neg, not_, pos
 from sys import version_info
 
 from yaml import safe_dump_all
@@ -191,11 +196,11 @@ def _generic_param2ast(param):
                     )
                     else ast.parse(_param["default"])
                 )
-            except SyntaxError:
+            except (SyntaxError, TypeError):
                 parsed_default = set_value(
                     _param["default"]
                     if code_quoted(_param["default"])
-                    else "```{}```".format(_param["default"])
+                    else "```{default}```".format(default=_param["default"])
                 )
 
             value = (
@@ -249,7 +254,9 @@ def find_ast_type(node, node_name=None, of_type=ClassDef):
         elif matching_nodes:
             return matching_nodes[0]
         else:
-            raise TypeError("No {!r} in AST".format(type(of_type).__name__))
+            raise TypeError(
+                "No {type_name!r} in AST".format(type_name=type(of_type).__name__)
+            )
     elif isinstance(node, AST):
         assert node_name is None or not hasattr(node, "name") or node.name == node_name
         return node
@@ -928,7 +935,9 @@ class RewriteAtQuery(NodeTransformer):
                 self.replacement_node = emit_arg(self.replacement_node)
             assert isinstance(
                 self.replacement_node, ast.arg
-            ), "Expected ast.arg got {!r}".format(type(self.replacement_node).__name__)
+            ), "Expected ast.arg got {replacement_node_name!r}".format(
+                replacement_node_name=type(self.replacement_node).__name__
+            )
 
             for arg_attr in "args", "kwonlyargs":
                 arg_l = getattr(node.args, arg_attr)
@@ -1052,9 +1061,10 @@ def infer_type_and_default(action, default, typ, required):
     elif hasattr(default, "__str__") and str(default) == "<required parameter>":
         # Special type that PyTorch uses & defines
         action, default, required, typ = None, None, True, default.__class__.__name__
-    elif isinstance(default, (list, tuple)):
+    elif isinstance(default, (list, tuple, set)):
+        # `set` actually won't marshall back properly as JSON/YAML doesn't support this type :(
         action, default, required, typ = _infer_type_and_default_for_list_or_tuple(
-            action, default, required
+            action, tuple(default) if isinstance(default, set) else default, required
         )
     elif isinstance(default, dict):
         typ = "loads"
@@ -1068,11 +1078,64 @@ def infer_type_and_default(action, default, typ, required):
             ("Any", "pickle.loads", "loads")
         ):
             typ = None
-    elif isinstance(default, type) or isfunction(default) or isclass(default):
+    elif any(
+        (
+            isinstance(default, type),
+            isfunction(default),
+            isclass(default),
+            contains(
+                frozenset(
+                    map(
+                        "tf.{}".format,
+                        (
+                            "qint16",
+                            "qint16_ref",
+                            "qint32",
+                            "qint32_ref",
+                            "qint8",
+                            "qint8_ref",
+                            "quint16",
+                            "quint16_ref",
+                            "quint8",
+                            "quint8_ref",
+                            "bfloat16",
+                            "bool",
+                            "complex128",
+                            "complex64",
+                            "double",
+                            "float16",
+                            "float32",
+                            "float64",
+                            "half",
+                            "int16",
+                            "int32",
+                            "int64",
+                            "int8",
+                            "qint16",
+                            "qint32",
+                            "qint8",
+                            "quint16",
+                            "quint8",
+                            "resource",
+                            "string",
+                            "uint16",
+                            "uint32",
+                            "uint64",
+                            "uint8",
+                            "variant",
+                        ),
+                    )
+                ),
+                repr(default),
+            ),
+        )
+    ):
         typ, default, required = "pickle.loads", pickle.dumps(default), False
     else:
         raise NotImplementedError(
-            "Parsing type {!s}, which contains {!r}".format(type(default), default)
+            "Parsing type {default_type!s}, which contains {default!r}".format(
+                default_type=type(default), default=default
+            )
         )
 
     return action, default, required, typ
@@ -1157,6 +1220,7 @@ def _parse_default_from_ast(action, default, required, typ):
     :returns: action, default, required, typ
     :rtype: ```Tuple[Optional[str], Optional[List[str]], bool, Optional[str]]```
     """
+
     if isinstance(default, (Constant, Expr, Str, Num)):
         default = get_value(default)
     # if type(default).__name__ in simple_types:
@@ -1180,6 +1244,7 @@ def _parse_default_from_ast(action, default, required, typ):
     #    required = "Optional" in (
     #        typ or iter(())
     #    )  # TODO: Work for `Union[None, AnyStr]` and `Any`
+
     return action, default, required, typ
 
 
@@ -1200,7 +1265,9 @@ def parse_to_scalar(node):
     elif isinstance(node, ast.AST):
         return _to_code(node).rstrip("\n")
     else:
-        raise NotImplementedError("Converting this to scalar: {!r}".format(node))
+        raise NotImplementedError(
+            "Converting this to scalar: {node!r}".format(node=node)
+        )
 
 
 # Construct from https://docs.sqlalchemy.org/en/13/core/type_basics.html#generic-types
@@ -1353,6 +1420,170 @@ def to_annotation(typ):
     )
 
 
+def to_type_comment(node):
+    """
+    Convert annotation to a type comment
+
+    :param node: AST node with a '.annotation' or Name or str
+    :type node: ```Union[Name, str, AnnAssign, arg, arguments]```
+
+    :returns: type_comment
+    :rtype: ```str```
+    """
+    return (
+        node.id
+        if isinstance(node, Name)
+        else node
+        if isinstance(node, str)
+        else _to_code(node.annotation).strip()
+    )
+
+
+def get_ass_where_name(node, name):
+    """
+    Find all `Assign`/`AnnAssign` in node body where id matches name
+
+    :param node: AST node with a '.body'
+    :type node: ```Union[Module, ClassDef, FunctionDef, If, Try, While, With, AsyncFor, AsyncFunctionDef, AsyncWith,
+                         ExceptHandler, Expression, For, IfExp, Interactive, Lambda ]```
+
+    :param name: Name to match (matches against `id` field of `Name`)
+    :type name: ```str```
+
+    :returns: Generator of all matches names (.value)
+    :rtype: ```Generator[Any]```
+    """
+    return (
+        get_value(_node)
+        for _node in node.body
+        if isinstance(_node, Assign)
+        and name
+        in frozenset(
+            map(attrgetter("id"), filter(rpartial(isinstance, Name), _node.targets))
+        )
+        or isinstance(_node, AnnAssign)
+        and get_value(_node.target) == name
+    )
+
+
+def del_ass_where_name(node, name):
+    """
+    Delete all `Assign`/`AnnAssign` in node body where id matches name
+
+    :param node: AST node with a '.body'
+    :type node: ```Union[Module, ClassDef, FunctionDef, If, Try, While, With, AsyncFor, AsyncFunctionDef, AsyncWith,
+                         ExceptHandler, Expression, For, IfExp, Interactive, Lambda ]```
+
+    :param name: Name to match (matches against `id` field of `Name`)
+    :type name: ```str```
+    """
+    node.body = list(
+        filter(
+            None,
+            (
+                None
+                if isinstance(_node, Assign)
+                and name
+                in frozenset(
+                    map(
+                        attrgetter("id"),
+                        filter(rpartial(isinstance, Name), _node.targets),
+                    )
+                )
+                or isinstance(_node, AnnAssign)
+                and _node.target == name
+                else _node
+                for _node in node.body
+            ),
+        )
+    )
+
+
+def merge_assignment_lists(node, name, unique_sort=True):
+    """
+    Merge multiple same-name lists within the body of a node into one, e.g., if you have multiple ```__all__```
+
+    :param node: AST node with a '.body'
+    :type node: ```Union[Module, ClassDef, FunctionDef, If, Try, While, With, AsyncFor, AsyncFunctionDef, AsyncWith,
+                         ExceptHandler, Expression, For, IfExp, Interactive, Lambda ]```
+
+    :param name: Name to match (matches against `id` field of `Name`)
+    :type name: ```str```
+
+    :param unique_sort: Whether to ensure its unique + sorted
+    :type unique_sort: ```bool```
+    """
+    asses = tuple(get_ass_where_name(node, name))
+
+    # if len(asses) < 2: return
+
+    # Could extract the `AnnAssign` stuff I suppose…
+
+    del_ass_where_name(node, name)
+    elts = map(
+        get_value,
+        chain.from_iterable(
+            map(
+                attrgetter("elts"),
+                asses,
+            )
+        ),
+    )
+    node.body.append(
+        Assign(
+            targets=[Name("__all__", Store())],
+            value=List(
+                ctx=Load(),
+                elts=list(
+                    map(set_value, (sorted(frozenset(elts)) if unique_sort else elts))
+                ),
+                expr=None,
+            ),
+            expr=None,
+            lineno=None,
+            **maybe_type_comment
+        )
+    )
+
+
+def merge_modules(mod0, mod1, remove_imports_from_second=True):
+    """
+    Merge modules (removing module docstring from mod1)
+
+    :param mod0: Module
+    :type mod0: ```Module```
+
+    :param mod1: Module
+    :type mod1: ```Module```
+
+    :param remove_imports_from_second: Whether to remove global imports from second module
+    :type remove_imports_from_second: ```bool```
+
+    :returns: Merged module (copy)
+    :rtype: ```Module```
+    """
+    mod1_body = (
+        mod1.body[1:]
+        if mod1.body and isinstance(get_value(mod1.body[0]), (Str, Constant))
+        else mod1.body
+    )
+
+    new_mod = deepcopy(mod0)
+
+    new_mod.body += (
+        list(
+            filterfalse(
+                rpartial(isinstance, (ImportFrom, Import)),
+                mod1_body,
+            )
+        )
+        if remove_imports_from_second
+        else deepcopy(mod1_body)
+    )
+
+    return new_mod
+
+
 NoneStr = "```(None)```" if PY_GTE_3_9 else "```None```"
 
 __all__ = [
@@ -1375,6 +1606,8 @@ __all__ = [
     "is_argparse_description",
     "it2literal",
     "maybe_type_comment",
+    "merge_assignment_lists",
+    "merge_modules",
     "node_to_dict",
     "param2argparse_param",
     "param2ast",
