@@ -6,7 +6,6 @@ from ast import (
     Assign,
     AsyncFunctionDef,
     ClassDef,
-    Expr,
     FunctionDef,
     NodeTransformer,
     get_docstring,
@@ -16,7 +15,13 @@ from collections import OrderedDict
 from operator import attrgetter
 
 from cdd import emit, parse
-from cdd.ast_utils import find_in_ast, set_arg, set_value, to_annotation
+from cdd.ast_utils import (
+    find_in_ast,
+    set_arg,
+    set_docstring,
+    to_annotation,
+    to_type_comment,
+)
 from cdd.docstring_parsers import parse_docstring
 from cdd.parser_utils import ir_merge
 
@@ -68,25 +73,23 @@ class DocTrans(NodeTransformer):
         self.whole_ast = whole_ast
         self.memoized = {}
 
-    '''
-    def generic_visit(self, node):
-        """
-        visits the `AST` node, if it could have a docstring pass it off to that handler
-
-        :param node: The AST node
-        :type node: ```AST```
-
-        :returns: Potentially changed AST node
-        :rtype: ```AST```
-        """
-        is_func, doc_str = isinstance(node, (AsyncFunctionDef, FunctionDef)), None
-        if is_func or isinstance(node, ClassDef):
-            node, doc_str = self._handle_node_with_docstring(node)
-            doc_str = ast.get_docstring(node)
-        if is_func:
-            node = self._handle_function(node, doc_str)
-        return super(DocTrans, self).generic_visit(node)
-    '''
+    # def generic_visit(self, node):
+    #     """
+    #     visits the `AST` node, if it could have a docstring pass it off to that handler
+    #
+    #     :param node: The AST node
+    #     :type node: ```AST```
+    #
+    #     :returns: Potentially changed AST node
+    #     :rtype: ```AST```
+    #     """
+    #     is_func, doc_str = isinstance(node, (AsyncFunctionDef, FunctionDef)), None
+    #     if is_func or isinstance(node, ClassDef):
+    #         node, doc_str = self._handle_node_with_docstring(node)
+    #         doc_str = ast.get_docstring(node)
+    #     if is_func:
+    #         node = self._handle_function(node, doc_str)
+    #     return super(DocTrans, self).generic_visit(node)
 
     def visit_AnnAssign(self, node):
         """
@@ -106,7 +109,7 @@ class DocTrans(NodeTransformer):
         return Assign(
             targets=[node.target],
             value=node.value,
-            type_comment=node.annotation,
+            type_comment=to_type_comment(node.annotation),
             lineno=None,
         )
 
@@ -124,7 +127,7 @@ class DocTrans(NodeTransformer):
         if self.type_annotations:
             assert len(node.targets) == 1
             return AnnAssign(
-                annotation=typ,
+                annotation=to_annotation(typ),
                 value=node.value,
                 type_comment=None,
                 lineno=None,
@@ -134,9 +137,27 @@ class DocTrans(NodeTransformer):
                 expr_target=None,
                 expr_annotation=None,
             )
-        # else:
-        #     node.type_comment = typ
-        # return node
+        else:
+            node.type_comment = typ
+        return node
+
+    def visit_Module(self, node):
+        """
+        visits the `Module`, potentially augmenting its docstring indentation
+
+        :param node: Module
+        :type node: ```Module```
+
+        :returns: Potentially changed Module
+        :rtype: ```Module```
+        """
+        # Clean might be wrong if the header is a license or other long-spiel documentation
+        doc_str = get_docstring(node, clean=True)
+        empty = doc_str is None
+        if not empty:
+            set_docstring("\n{doc_str}\n".format(doc_str=doc_str), empty, node)
+        node.body = list(map(self.visit, node.body))
+        return node
 
     def visit_FunctionDef(self, node):
         """
@@ -165,7 +186,8 @@ class DocTrans(NodeTransformer):
             if isinstance(node, Assign)
             else (node.target, {"typ": node.annotation or node.type_comment})
         )
-        # if not hasattr(node, "_location"): return typ_dict["typ"]
+        if not hasattr(node, "_location"):
+            return typ_dict["typ"]
         search = node._location[:-1]
         search_str = ".".join(search)
 
@@ -207,17 +229,21 @@ class DocTrans(NodeTransformer):
         ir = parse_docstring(doc_str)
         ir_merge(ir, parse.function(node))
         ir["name"] = node.name
+        indent_level = max(
+            len(node._location) - 1, 1
+        )  # function docstrings always have at least 1 indent level
         _doc_str = emit.docstring(
             ir,
             emit_types=not self.type_annotations,
+            emit_default_doc=False,
             docstring_format=self.docstring_format,
-            indent_level=len(node._location) - 1,
+            indent_level=indent_level,
         )
         if _doc_str.isspace():
             if doc_str is not None:
                 del node.body[0]
         else:
-            node.body.insert(0, Expr(set_value(_doc_str)))
+            set_docstring(_doc_str, False, node)
         if self.type_annotations:
             # Add annotations
             if ir["params"]:
@@ -244,9 +270,7 @@ class DocTrans(NodeTransformer):
             node.args.args = list(map(set_arg, map(attrgetter("arg"), node.args.args)))
             node.returns = None
 
-        # Odd that this is required; I guess it's because walk/visit order isn't guaranteed depth first…
         node.body = list(map(self.visit, node.body))
-
         return node
 
 
@@ -267,4 +291,4 @@ def clear_annotation(node):
     return node
 
 
-__all__ = ["DocTrans", "has_type_annotations"]
+__all__ = ["DocTrans", "clear_annotation", "has_type_annotations"]
