@@ -1,6 +1,7 @@
 """
 Pure utils for pure functions. For the same input will always produce the same input_str.
 """
+
 import typing
 from ast import Name, Str
 from collections import deque
@@ -11,14 +12,14 @@ from inspect import getmodule
 from itertools import chain, count, zip_longest
 from keyword import iskeyword
 from operator import attrgetter, eq
-from os import environ, path
+from os import environ, extsep, path
 from pprint import PrettyPrinter
 from sys import version_info
 from textwrap import fill as _fill
 from textwrap import indent
-from typing import Dict, FrozenSet, Optional, Union
+from typing import Callable, Dict, FrozenSet, Optional, Tuple, Union
 
-pp = PrettyPrinter(indent=4, width=100).pprint
+pp: Callable[[object], None] = PrettyPrinter(indent=4, width=100).pprint
 tab: str = " " * 4
 simple_types: Dict[Optional[str], Union[int, float, complex, str, bool, None]] = {
     "int": 0,
@@ -39,7 +40,7 @@ def rpartial(func, *args):
     return lambda *a: func(*(a + args))
 
 
-def identity(*args):
+def identity(*args, **kwargs):
     """
     Identity function
 
@@ -52,11 +53,13 @@ def identity(*args):
     return args[0] if len(args) == 1 else args
 
 
-_python_major_minor: typing.Tuple[int, int] = version_info[:2]
+_python_major_minor: Tuple[int, int] = version_info[:2]
 PY3_8: bool = _python_major_minor == (3, 8)
 PY_GTE_3_8: bool = _python_major_minor >= (3, 8)
 PY_GTE_3_9: bool = _python_major_minor >= (3, 9)
 
+ENCODING = "# -*- coding: utf-8 -*-"
+
 none_types = None, "None", "```(None)```" if PY_GTE_3_9 else "```None```"
 
 _ABERRANT_PLURAL_MAP: Dict[str, str] = {
@@ -312,26 +315,33 @@ def quote(s, mark='"'):
     """
     Quote the input string if it's not already quoted
 
-    :param s: Input string
-    :type s: ```str```
+    :param s: Input string or literal or None
+    :type s: ```Union[str, float, complex, int, None]```
 
     :param mark: Quote mark to wrap with
     :type mark: ```str```
 
-    :returns: Quoted string
-    :rtype: ```str```
+    :returns: Quoted string or input (if input is not str)
+    :rtype: ```Union[str, float, complex, int, None]```
     """
+    very_simple_types = type(None), int, float, complex
     s = (
         s
-        if isinstance(s, (str, type(None)))
+        if isinstance(s, (str, *very_simple_types))
         else s.s
         if isinstance(s, Str)
         else s.id
         if isinstance(s, Name)
-        else s.value
+        else getattr(s, "value", s)
     )
     # ^ Poor man's `get_value`
-    if s is None or len(s) == 0 or s[0] == s[-1] and s[0] in frozenset(("'", '"')):
+    if (
+        isinstance(s, very_simple_types)
+        or len(s) == 0
+        or len(s) > 1
+        and s[0] == s[-1]
+        and s[0] in frozenset(("'", '"'))
+    ):
         return s
     return "{mark}{s}{mark}".format(mark=mark, s=s)
 
@@ -353,7 +363,7 @@ def assert_equal(a, b, cmp=eq):
     :rtype: ```Literal[True]```
     """
     if not cmp(a, b):
-        raise AssertionError("{!r} != {!r}".format(a, b))
+        raise AssertionError("{a!r} != {b!r}".format(a=a, b=b))
     return True
 
 
@@ -493,6 +503,8 @@ BUILTIN_TYPES = (
     | frozenset(("int", "float", "str", "dict", "list", "tuple"))
 )
 
+INIT_FILENAME = "__init__{extsep}py".format(extsep=extsep)
+
 
 def code_quoted(s):
     """
@@ -600,7 +612,7 @@ def filename_from_mod_or_filename(mod_or_filename):
     ).origin
 
 
-def emit_separating_tabs(s, indent_level=1):
+def emit_separating_tabs(s, indent_level=1, run_per_line=str.lstrip):
     """
     Emit a separating tab between paragraphs
 
@@ -610,18 +622,88 @@ def emit_separating_tabs(s, indent_level=1):
     :param indent_level: docstring indentation level whence: 0=no_tabs, 1=one tab; 2=two tabs
     :type indent_level: ```int```
 
+    :param run_per_line: Run this function per line
+    :type run_per_line: ```Callable[[str], str]```
+
+    :returns: Original string with a separating tab between paragraphs, & possibly addition indentation on other lines
+    :rtype: ```str```
     """
     sep = tab * indent_level
     return "\n{sep}{}\n{sep}".format(
-        "\n".join(
-            map(lambda line: sep if len(line) == 0 else line, s.splitlines())
-        ).lstrip(),
+        run_per_line(
+            "\n".join(map(lambda line: sep if len(line) == 0 else line, s.splitlines()))
+        ),
         sep=sep,
     )
 
 
+def set_attr(obj, key, val):
+    """
+    Sets the named attribute on the given object to the specified value.
+
+    set_attr(x, 'y', v) is equivalent to ``x.y = v; return x''
+
+    :param obj: An object
+    :type obj: ```Any```
+
+    :param key: A key
+    :type key: ```str```
+
+    :param val: A value
+    :type val: ```Any```
+
+    :returns: The modified `obj`
+    :rtype: ```Any```
+    """
+    setattr(obj, key, val)
+    return obj
+
+
+def set_item(obj, key, val):
+    """
+    Sets the item on the given object to the specified value.
+
+    set_item(x, 'y', v) is equivalent to ``x[y] = v; return x''
+
+    :param obj: An object
+    :type obj: ```Any```
+
+    :param key: A key
+    :type key: ```Union[str, int]```
+
+    :param val: A value
+    :type val: ```Any```
+
+    :returns: The modified `obj`
+    :rtype: ```Any```
+    """
+    obj[key] = val
+    return obj
+
+
+def no_magic_dir2attr(p_object):
+    """
+    Dictionary of `dir` without the __ prefix magics (also without _ prefix)
+    return the names comprising (some of) the attributes
+    of the given object, and of attributes reachable from it.
+
+    :param p_object: Object
+    :type p_object: ```Any```
+
+    :returns: Dict of name to attribute value
+    :rtype: ```dict```
+    """
+    return {
+        attr: getattr(p_object, attr)
+        for attr in dir(p_object)
+        if not attr.startswith("_")
+    }
+
+
 __all__ = [
     "BUILTIN_TYPES",
+    "ENCODING",
+    "INIT_FILENAME",
     "PY3_8",
     "PY_GTE_3_8",
     "PY_GTE_3_9",
@@ -638,6 +720,7 @@ __all__ = [
     "indent_all_but_first",
     "location_within",
     "lstrip_namespace",
+    "no_magic_dir2attr",
     "none_types",
     "paren_wrap_code",
     "pluralise",
@@ -646,6 +729,8 @@ __all__ = [
     "reindent",
     "rpartial",
     "sanitise",
+    "set_attr",
+    "set_item",
     "simple_types",
     "strip_split",
     "tab",
