@@ -1,11 +1,10 @@
 """ Tests for exmod subcommand """
-
 from ast import ClassDef
 from functools import partial
 from io import StringIO
 from itertools import chain, groupby
 from operator import add, itemgetter
-from os import environ, mkdir, path
+from os import environ, listdir, mkdir, path, walk
 from os.path import extsep
 from subprocess import DEVNULL, call
 from sys import executable, platform
@@ -27,40 +26,168 @@ from cdd.tests.utils_for_tests import unittest_main
 class TestExMod(TestCase):
     """Test class for exmod.py"""
 
+    def test_exmod(self) -> None:
+        """Tests `exmod`"""
+
+        try:
+            with TemporaryDirectory(
+                prefix="gold", suffix="gold"
+            ) as existent_module_dir, TemporaryDirectory(
+                prefix="gen", suffix="gen"
+            ) as new_module_dir:
+                self._create_fs(existent_module_dir)
+                # sys.path.insert(0, existent_module_dir)
+                self._pip(["install", "."], existent_module_dir)
+
+                exmod(
+                    module=self.module_name,
+                    emit_name="class",
+                    blacklist=tuple(),
+                    whitelist=tuple(),
+                    output_directory=new_module_dir,
+                    dry_run=False,
+                )
+                self._check_emission(new_module_dir)
+        finally:
+            # sys.path.remove(existent_module_dir)
+            self._pip(["uninstall", "-y", self.module_name])
+
     def test_exmod_blacklist(self) -> None:
         """Tests `exmod` blacklist"""
 
-        with TemporaryDirectory() as tempdir, self.assertRaises(NotImplementedError):
-            exmod(
-                module="unittest",
-                emit_name=None,
-                blacklist=("unittest.TestCase",),
-                whitelist=tuple(),
-                output_directory=tempdir,
-                dry_run=False,
-            )
+        try:
+            with TemporaryDirectory(
+                prefix="gold", suffix="gold"
+            ) as existent_module_dir, TemporaryDirectory(
+                prefix="gen", suffix="gen"
+            ) as new_module_dir:
+                self._create_fs(existent_module_dir)
+                self._pip(["install", "."], existent_module_dir)
+                exmod(
+                    module=self.module_name,
+                    emit_name="class",
+                    blacklist=(".".join((self.module_name,) * 2),),
+                    whitelist=tuple(),
+                    output_directory=new_module_dir,
+                    dry_run=False,
+                )
+                self.assertListEqual(listdir(new_module_dir), [])
+        finally:
+            self._pip(["uninstall", "-y", self.module_name])
 
     def test_exmod_whitelist(self) -> None:
         """Tests `exmod` whitelist"""
 
-        with TemporaryDirectory() as tempdir, self.assertRaises(NotImplementedError):
-            exmod(
-                module="unittest",
-                emit_name=None,
-                blacklist=tuple(),
-                whitelist=("unittest.TestCase",),
-                output_directory=tempdir,
-                dry_run=False,
-            )
+        try:
+            with TemporaryDirectory(
+                prefix="gold", suffix="gold"
+            ) as existent_module_dir, TemporaryDirectory(
+                prefix="gen", suffix="gen"
+            ) as new_module_dir:
+                self._create_fs(existent_module_dir)
+                self._pip(["install", "."], existent_module_dir)
+                existent_module_name = path.basename(existent_module_dir)
+                exmod(
+                    module=self.module_name,
+                    emit_name="class",
+                    blacklist=tuple(),
+                    whitelist=(".".join((existent_module_name,) * 2),),
+                    output_directory=new_module_dir,
+                    dry_run=False,
+                )
+
+                gen, gold = map(
+                    sorted,
+                    (
+                        map(
+                            lambda p: (
+                                lambda _p: path.join("gold", _p.partition(path.sep)[2])
+                                if _p.startswith("gold") and _p.count("gold") == 2
+                                else _p
+                            )(p.partition(path.sep)[2]),
+                            (
+                                path.join(dirpath, filename)[
+                                    len(new_module_dir + path.sep) :
+                                ]
+                                for (dirpath, dirnames, filenames) in walk(
+                                    new_module_dir
+                                )
+                                for filename in filenames
+                            ),
+                        ),
+                        (
+                            INIT_FILENAME,
+                            *chain.from_iterable(
+                                map(
+                                    lambda i: map(
+                                        partial(
+                                            path.join,
+                                            self.module_hierarchy[i][1],
+                                        ),
+                                        (
+                                            "{name}{sep}py".format(
+                                                name=self.module_hierarchy[i][0],
+                                                sep=extsep,
+                                            ),
+                                            INIT_FILENAME,
+                                        ),
+                                    ),
+                                    range(len(self.module_hierarchy)),
+                                ),
+                            ),
+                        ),
+                    ),
+                )
+                all_tests_running = len(gold) == 7
+                if all_tests_running:
+                    gold = list(
+                        chain.from_iterable(
+                            (
+                                gold[:1],
+                                chain.from_iterable(
+                                    map(
+                                        lambda p: (p, p),
+                                        map(
+                                            partial(path.join, "gold", "parent_dir"),
+                                            (
+                                                INIT_FILENAME,
+                                                *map(
+                                                    partial(path.join, "child_dir"),
+                                                    (
+                                                        INIT_FILENAME,
+                                                        "child.py",
+                                                        path.join(
+                                                            "grandchild_dir",
+                                                            INIT_FILENAME,
+                                                        ),
+                                                        path.join(
+                                                            "grandchild_dir",
+                                                            "grandchild.py",
+                                                        ),
+                                                    ),
+                                                ),
+                                                "parent.py",
+                                            ),
+                                        ),
+                                    )
+                                ),
+                                gold[1:],
+                            )
+                        )
+                    )
+
+                self.assertListEqual(gen, gold)
+        finally:
+            self._pip(["uninstall", "-y", self.module_name])
 
     def test_exmod_module_directory(self) -> None:
         """Tests `exmod` module whence directory"""
 
-        with TemporaryDirectory() as tempdir, self.assertRaises(NotImplementedError):
+        with TemporaryDirectory() as tempdir, self.assertRaises(ModuleNotFoundError):
             exmod(
                 module=tempdir,
                 emit_name="cjvxclkvjclxkjvlcx",
-                blacklist=["foo", "bar"],
+                blacklist=tuple(),
                 whitelist=tuple(),
                 output_directory=path.join(tempdir, "nonexistent"),
                 dry_run=False,
@@ -81,7 +208,176 @@ class TestExMod(TestCase):
                 dry_run=False,
             )
 
-    def create_fs(self, tempdir):
+    def test_exmod_dry_run(self) -> None:
+        """Tests `exmod` dry_run"""
+
+        try:
+            with TemporaryDirectory(
+                prefix="gold", suffix="gold"
+            ) as existent_module_dir, TemporaryDirectory(
+                prefix="gen", suffix="gen"
+            ) as new_module_dir:
+                self._create_fs(existent_module_dir)
+                self._pip(["install", "."], existent_module_dir)
+
+                with patch("sys.stdout", new_callable=StringIO) as f:
+                    exmod(
+                        module=self.module_name,
+                        emit_name="class",
+                        blacklist=tuple(),
+                        whitelist=tuple(),
+                        output_directory=new_module_dir,
+                        dry_run=True,
+                    )
+                    r = f.getvalue()
+
+                result = dict(
+                    map(
+                        lambda k_v: (
+                            k_v[0],
+                            tuple(
+                                sorted(
+                                    set(
+                                        map(
+                                            partial(
+                                                relative_filename,
+                                                remove_hints=(
+                                                    (
+                                                        lambda directory: "{directory}{sep}".format(
+                                                            directory=unquote(
+                                                                repr(directory)
+                                                            ),
+                                                            sep=path.sep,
+                                                        )
+                                                        if platform == "win32"
+                                                        else directory
+                                                    )(
+                                                        path.join(
+                                                            new_module_dir,
+                                                            path.basename(
+                                                                new_module_dir
+                                                            ),
+                                                        ),
+                                                    ),
+                                                ),
+                                            ),
+                                            map(unquote, map(itemgetter(1), k_v[1])),
+                                        )
+                                    )
+                                )
+                            ),
+                        ),
+                        groupby(
+                            map(rpartial(str.split, "\t", 2), sorted(r.splitlines())),
+                            key=itemgetter(0),
+                        ),
+                    )
+                )
+
+                all_tests_running = len(result["write"]) == 1
+
+                key_counts = (
+                    (("mkdir", 10), ("touch", 4), ("write", 1))
+                    if all_tests_running
+                    else (("mkdir", 7), ("touch", 4), ("write", 4))
+                )
+
+                for key, count in key_counts:
+                    self.assertEqual(count, len(result[key]), key)
+
+                gold_module_name = next(
+                    map(
+                        lambda p: p.partition(path.sep)[0],
+                        filter(rpartial(str.startswith, "gold"), result["mkdir"]),
+                    ),
+                    path.basename(self.gold_dir),
+                )
+                gold = {
+                    k: tuple(map(unquote, map(repr, v)))
+                    for k, v in {
+                        "mkdir": chain.from_iterable(
+                            (
+                                (new_module_dir,),
+                                map(
+                                    partial(path.join, gold_module_name),
+                                    (
+                                        self.module_hierarchy[0][1],
+                                        self.module_hierarchy[1][1],
+                                        self.module_hierarchy[2][1],
+                                    ),
+                                )
+                                if all_tests_running
+                                else iter(()),
+                                (
+                                    self.module_hierarchy[0][1],
+                                    self.module_hierarchy[1][1],
+                                    path.join(
+                                        self.module_hierarchy[1][1],
+                                        self.module_hierarchy[1][0],
+                                    ),
+                                    self.module_hierarchy[2][1],
+                                    path.join(
+                                        self.module_hierarchy[2][1],
+                                        self.module_hierarchy[2][0],
+                                    ),
+                                    path.join(
+                                        self.module_hierarchy[0][1],
+                                        self.module_hierarchy[0][0],
+                                    ),
+                                ),
+                            )
+                        ),
+                        "touch": (
+                            INIT_FILENAME,
+                            *map(
+                                rpartial(path.join, INIT_FILENAME),
+                                (
+                                    self.module_hierarchy[0][1],
+                                    self.module_hierarchy[1][1],
+                                    self.module_hierarchy[2][1],
+                                ),
+                            ),
+                        ),
+                        "write": (
+                            lambda write_block: tuple(write_block[:1])
+                            if all_tests_running
+                            else write_block
+                        )(
+                            (
+                                INIT_FILENAME,
+                                path.join(
+                                    self.module_hierarchy[1][1],
+                                    "{name}{extsep}py".format(
+                                        name=self.module_hierarchy[1][0],
+                                        extsep=extsep,
+                                    ),
+                                ),
+                                path.join(
+                                    self.module_hierarchy[2][1],
+                                    "{name}{extsep}py".format(
+                                        name=self.module_hierarchy[2][0],
+                                        extsep=extsep,
+                                    ),
+                                ),
+                                path.join(
+                                    self.module_hierarchy[0][1],
+                                    "{name}{extsep}py".format(
+                                        name=self.module_hierarchy[0][0],
+                                        extsep=extsep,
+                                    ),
+                                ),
+                            )
+                        ),
+                    }.items()
+                }
+
+                self.assertDictEqual(result, gold)
+
+                self._check_emission(new_module_dir, dry_run=True)
+        finally:
+            self._pip(["uninstall", "-y", self.module_name])
+
+    def _create_fs(self, tempdir):
         """
         Populate filesystem from `tempdir` root with module hierarchy &etc. for later exposure (exmod)
 
@@ -194,7 +490,7 @@ class TestExMod(TestCase):
 
         return tempdir
 
-    def check_emission(self, tempdir, dry_run=False):
+    def _check_emission(self, tempdir, dry_run=False):
         """
         Confirm whether emission conforms to gen by verifying their IRs are equivalent
 
@@ -269,217 +565,6 @@ class TestExMod(TestCase):
             "EXIT_SUCCESS not reached",
         )
 
-    def test_exmod(self) -> None:
-        """Tests `exmod`"""
-
-        try:
-            with TemporaryDirectory(
-                prefix="gold", suffix="gold"
-            ) as existent_module_dir, TemporaryDirectory(
-                prefix="gen", suffix="gen"
-            ) as new_module_dir:
-                self.create_fs(existent_module_dir)
-                self._pip(["install", "."], existent_module_dir)
-                exmod(
-                    module=self.module_name,
-                    emit_name="class",
-                    blacklist=tuple(),
-                    whitelist=tuple(),
-                    output_directory=new_module_dir,
-                    dry_run=False,
-                )
-                self.check_emission(new_module_dir)
-        finally:
-            self._pip(["uninstall", "-y", self.module_name])
-
-    def test_exmod_dry_run(self) -> None:
-        """Tests `exmod` dry_run"""
-
-        try:
-            with TemporaryDirectory(
-                prefix="gold", suffix="gold"
-            ) as existent_module_dir, TemporaryDirectory(
-                prefix="gen", suffix="gen"
-            ) as new_module_dir:
-                self.create_fs(existent_module_dir)
-                self._pip(["install", "."], existent_module_dir)
-
-                with patch("sys.stdout", new_callable=StringIO) as f:
-                    exmod(
-                        module=self.module_name,
-                        emit_name="class",
-                        blacklist=tuple(),
-                        whitelist=tuple(),
-                        output_directory=new_module_dir,
-                        dry_run=True,
-                    )
-                    r = f.getvalue()
-
-                result = dict(
-                    map(
-                        lambda k_v: (
-                            k_v[0],
-                            tuple(
-                                sorted(
-                                    set(
-                                        map(
-                                            partial(
-                                                relative_filename,
-                                                remove_hints=(
-                                                    (
-                                                        lambda directory: "{directory}{sep}".format(
-                                                            directory=unquote(
-                                                                repr(directory)
-                                                            ),
-                                                            sep=path.sep,
-                                                        )
-                                                        if platform == "win32"
-                                                        else directory
-                                                    )(
-                                                        path.join(
-                                                            new_module_dir,
-                                                            path.basename(
-                                                                new_module_dir
-                                                            ),
-                                                        ),
-                                                    ),
-                                                ),
-                                            ),
-                                            map(unquote, map(itemgetter(1), k_v[1])),
-                                        )
-                                    )
-                                )
-                            ),
-                        ),
-                        groupby(
-                            map(rpartial(str.split, "\t", 2), sorted(r.splitlines())),
-                            key=itemgetter(0),
-                        ),
-                    )
-                )
-
-                all_tests_running = len(result["write"]) == 7
-
-                key_counts = (
-                    (("mkdir", 10), ("touch", 4), ("write", 7))
-                    if all_tests_running
-                    else (("mkdir", 7), ("touch", 4), ("write", 4))
-                )
-
-                for key, count in key_counts:
-                    self.assertEqual(count, len(result[key]), key)
-
-                gold_module_name = next(
-                    map(
-                        lambda p: p.partition(path.sep)[0],
-                        filter(rpartial(str.startswith, "gold"), result["write"]),
-                    ),
-                    "",
-                )
-                gold = {
-                    k: tuple(map(unquote, map(repr, v)))
-                    for k, v in {
-                        "mkdir": chain.from_iterable(
-                            (
-                                (new_module_dir,),
-                                (
-                                    path.join(
-                                        gold_module_name, self.module_hierarchy[0][1]
-                                    ),
-                                    path.join(
-                                        gold_module_name, self.module_hierarchy[1][1]
-                                    ),
-                                    path.join(
-                                        gold_module_name, self.module_hierarchy[2][1]
-                                    ),
-                                )
-                                if all_tests_running
-                                else iter(()),
-                                (
-                                    self.module_hierarchy[0][1],
-                                    self.module_hierarchy[1][1],
-                                    path.join(
-                                        self.module_hierarchy[1][1],
-                                        self.module_hierarchy[1][0],
-                                    ),
-                                    self.module_hierarchy[2][1],
-                                    path.join(
-                                        self.module_hierarchy[2][1],
-                                        self.module_hierarchy[2][0],
-                                    ),
-                                    path.join(
-                                        self.module_hierarchy[0][1],
-                                        self.module_hierarchy[0][0],
-                                    ),
-                                ),
-                            )
-                        ),
-                        "touch": (
-                            INIT_FILENAME,
-                            path.join(
-                                self.module_hierarchy[0][1],
-                                INIT_FILENAME,
-                            ),
-                            path.join(
-                                self.module_hierarchy[1][1],
-                                INIT_FILENAME,
-                            ),
-                            path.join(
-                                self.module_hierarchy[2][1],
-                                INIT_FILENAME,
-                            ),
-                        ),
-                        "write": (
-                            lambda write_block: tuple(
-                                sorted(
-                                    chain.from_iterable(
-                                        (
-                                            map(
-                                                partial(path.join, gold_module_name),
-                                                write_block[1:],
-                                            ),
-                                            write_block,
-                                        )
-                                    )
-                                )
-                            )
-                            if all_tests_running
-                            else write_block
-                        )(
-                            (
-                                INIT_FILENAME,
-                                path.join(
-                                    self.module_hierarchy[1][1],
-                                    "{name}{extsep}py".format(
-                                        name=self.module_hierarchy[1][0],
-                                        extsep=extsep,
-                                    ),
-                                ),
-                                path.join(
-                                    self.module_hierarchy[2][1],
-                                    "{name}{extsep}py".format(
-                                        name=self.module_hierarchy[2][0],
-                                        extsep=extsep,
-                                    ),
-                                ),
-                                path.join(
-                                    self.module_hierarchy[0][1],
-                                    "{name}{extsep}py".format(
-                                        name=self.module_hierarchy[0][0],
-                                        extsep=extsep,
-                                    ),
-                                ),
-                            )
-                        ),
-                    }.items()
-                }
-
-                self.assertDictEqual(result, gold)
-
-                self.check_emission(new_module_dir, dry_run=True)
-        finally:
-            self._pip(["uninstall", "-y", self.module_name])
-
     maxDiff = None
 
 
